#
# Copyright (c) Microsoft Corporation.  All rights reserved.
#

from ..RxSerializable import RxSerializable
from ..utils.RxOptions import RxOptions


class RxFileSystem(RxSerializable):
    """
    .. rxtitle::

        Class generator for file data source objects

    .. rxdescription::

        Creates a data source connection to a file on the native file system.

    .. rxkeywords:: file, datasource
    """
    def file_system_type(self) -> str:
        return self._file_system_type

    def __init__(self,
                 file_system_type="native"):
        self._file_system_type = file_system_type

        self._to_serialize = {}
        self.add_serialization_property("fileSystemType", self._file_system_type)

    def add_serialization_property(self, property_name, value):
        self._to_serialize[property_name] = value

    @property
    def to_serialize(self) -> dict:
        return self._to_serialize


class RxNativeFileSystem(RxFileSystem):
    """
    .. rxtitle::
        
        Class generator for the native file system

    .. rxdescription::
    
        Main generator class for objects repreresenting the native file system.

    :return:
   
        An RxNativeFileSystem file system object. This object may be used in :py:func:`RxOptions <revoscalepy.RxOptions>`, :py:func:`RxTextData <revoscalepy.RxTextData>`, or :py:func:`RxXdfData <revoscalepy.RxXdfData>` to set the file system.
        
    .. rxkeywords:: filesystem native
    
    .. rxseealso::
        :py:func:`RxOptions <revoscalepy.RxOptions>`
        :py:func:`RxTextData <revoscalepy.RxTextData>`
        :py:func:`RxXdfData <revoscalepy.RxXdfData>`
        :py:func:`RxFileSystem <revoscalepy.RxFileSystem>`
        
    .. rxexample::
        from revoscalepy import RxNativeFileSystem
        fs = RxNativeFileSystem()
        print(fs.file_system_type())
        
    """
    def __init__(self):
        super().__init__(file_system_type="native")


class RxHdfsFileSystem(RxFileSystem):
    """
    .. rxtitle::

        Class generator for an HDFS file system object

    .. rxdescription::

        Main generator class for RxHdfsFileSystem.

    :return:

        An RxHdfsFileSystem file system object.
        This object may be used in :py:func:`RxOptions <revoscalepy.RxOptions>`,
        :py:func:`RxTextData <revoscalepy.RxTextData>`,
        :py:func:`RxXdfData <revoscalepy.RxXdfData>`,
        :py:func:`RxParquetData <revoscalepy.RxParquetData>`,
        or :py:func:`RxOrcData <revoscalepy.RxOrcData>` to set the file system.

    .. rxkeywords:: filesystem hdfs

    .. rxseealso::
        :py:func:`RxOptions <revoscalepy.RxOptions>`
        :py:func:`RxTextData <revoscalepy.RxTextData>`
        :py:func:`RxXdfData <revoscalepy.RxXdfData>`
        :py:func:`RxParquet <revoscalepy.RxParquetData>`
        :py:func:`RxOrcData <revoscalepy.RxOrcData>`
        :py:func:`RxFileSystem <revoscalepy.RxFileSystem>`

    .. rxexample::
        from revoscalepy import RxHdfsFileSystem
        fs = RxHdfsFileSystem()
        print(fs.file_system_type())
        from revoscalepy import RxXdfData
        xdf_data = RxXdfData("/tmp/hdfs_file", file_system = fs)
    """

    def host_name(self) -> str:
        return self._host_name

    def port(self) -> int:
        return self._port

    def __init__(self,
                 host_name=None,
                 port=None):

        if host_name==None or len(host_name)==0:
            host_name=RxOptions.get_option("hdfsHost")

        if port==None:
            port=RxOptions.get_option("hdfsPort")

        if host_name==None or len(host_name) == 0:
            raise ValueError("'host_name' must be specified for an HDFS file system.")
        if port is None:
            raise ValueError("'port' must be specified for an HDFS file system.")

        if host_name.endswith('/') and not host_name.endswith('//') and len(host_name)>1:
            host_name = host_name[:-1]

        super().__init__(file_system_type="hdfs")
        self._host_name = host_name
        self._port = int(port)

        self.add_serialization_property("hostName", self._host_name)
        self.add_serialization_property("port", self._port)
