from ..RxSerializable import rx_native_call
from .RxReadObject import rx_read_object
from revoscalepy.utils import telemetry_capture_call
import warnings

def rx_unserialize_model(serialized_model):
    """
    .. rxtitle::

        Un-serialize a serialized model

    .. rxdescription::

        Un-serialize the given python model.

    :param serialized_model: A serialized Python model. Supported models
        are "rx_logit", "rx_lin_mod", "rx_dtree", "rx_btrees", "rx_dforest" and MicrosoftML models.

    :return: a Python model

    .. rxkeywords:: unserialization

    .. rxexample::
        import os
        from revoscalepy import RxOptions, RxXdfData, rx_lin_mod, RxOdbcData
        from revoscalepy import rx_serialize_model, rx_write_object, rx_read_object, rx_unserialize_model, rx_delete_object
        sample_data_path = RxOptions.get_option("sampleDataDir")
        ds = RxXdfData(os.path.join(sample_data_path, "AirlineDemoSmall.xdf"))
        linmod = rx_lin_mod("ArrDelay~DayOfWeek", ds)
        s_linmod = rx_serialize_model(linmod, realtime_scoring_only = False)
        us_linmod = rx_unserialize_model(s_linmod)

        # Writing and reading to/from a ODBC data source
        ## Set up a connection
        connStr = "Driver=SQL Server;Server=.;Database=RevoTestDb;Trusted_Connection=True;"
        data_src = RxOdbcData(table="modelsObjects", connection_string=connStr)
        ## Store the serialized model and read it from the database.
        rx_write_object(data_src, key="serialized_linmod", value=s_linmod, serialize=False)
        s_linmod_odbc = rx_read_object(data_src, key="serialized_linmod", deserialize=False)
        linmod_odbc = rx_unserialize_model(s_linmod_odbc)
    """
    telemetry_capture_call()

    if serialized_model is None:
        raise ValueError("serialized model cannot be none.")

    # Check if the model is serialized.
    if (type(serialized_model) == bytes):

        param_dict = ({'runtimeId': '0e472e9c-2635-4afc-8a44-74cfd49695c6',
                       'model': {"":[serialized_model]},
                       'metadata': None})
    else:
        raise ValueError("Function can only un-serialize a serialized model")

    function_name = "UnserializeModel" 
    unserialized_model_info = rx_native_call(functionname=function_name, params=param_dict)

    if unserialized_model_info is not None :
        if 'pymodel' in unserialized_model_info.keys() : 
            return rx_read_object(unserialized_model_info["pymodel"])
        else : 
            warnings.warn("Couldn't un-serialize: check if the serialized model is a revoscalepy model.")

    return unserialized_model_info
